@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qcheckbox.h>
@@ -279,8 +279,8 @@ bool MrmlPart::openURL( const KURL& url )
                                   "specified. Do you want to configure them "
                                   "now?"),
                                             i18n("Configuration Missing"),
-                                            KStdGuiItem::yes(),
-                                            KStdGuiItem::no(),
+                                            i18n("Configure"),
+                                            i18n("Do Not Configure"),
                                             "kmrml_ask_configure_gift" )
                  == KMessageBox::Yes )
             {
