@@ -1,6 +1,6 @@
 
 /*
-   Copyright (c) 2003-2004 Clarence Dang <dang@kde.org>
+   Copyright (c) 2003,2004,2005 Clarence Dang <dang@kde.org>
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -461,7 +461,7 @@ bool kpDocument::lossyPromptContinue (const QPixmap &pixmap,
                 i18n ("<qt><p>Saving the image at the low color depth of %1-bit"
                         " may result in the loss of color information."
 
-                        " Any transparency will also be removed."
+                        " Any transparency will also be removed.</p>"
 
                         "<p>Are you sure you want to save at this color depth?</p></qt>")
                     .arg (saveOptions.colorDepth ()),
@@ -1300,8 +1300,17 @@ bool kpDocument::selectionCopyOntoDocument (bool useTransparentPixmap)
     if (!boundingRect.isValid ())
         return false;
 
-    paintPixmapAt (useTransparentPixmap ? sel->transparentPixmap () : sel->opaquePixmap (),
-                   boundingRect.topLeft ());
+    if (!sel->isText ())
+    {
+        paintPixmapAt (useTransparentPixmap ? sel->transparentPixmap () : sel->opaquePixmap (),
+                       boundingRect.topLeft ());
+    }
+    else
+    {
+        // (for antialiasing with background)
+        sel->paint (m_pixmap, rect ());
+    }
+
     return true;
 }
 
