@@ -1,6 +1,6 @@
 
 /*
-   Copyright (c) 2003-2004 Clarence Dang <dang@kde.org>
+   Copyright (c) 2003,2004,2005 Clarence Dang <dang@kde.org>
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,7 @@
 
 class QPainter;
 class QPoint;
+class QPopupMenu;
 class QRect;
 class QSize;
 class QStringList;
@@ -68,7 +69,6 @@ class kpCommandHistory;
 class kpDocument;
 class kpDocumentMetaInfo;
 class kpDocumentSaveOptions;
-class kpView;
 class kpViewManager;
 class kpViewScrollableContainer;
 class kpSelection;
@@ -77,9 +77,11 @@ class kpSingleKeyTriggersAction;
 class kpSqueezedTextLabel;
 class kpTextStyle;
 class kpThumbnail;
+class kpThumbnailView;
 class kpTool;
 class kpToolText;
 class kpToolToolBar;
+class kpZoomedView;
 
 
 class kpMainWindow : public KMainWindow
@@ -111,6 +113,7 @@ private:
 
     bool m_configThumbnailShown;
     QRect m_configThumbnailGeometry;
+    bool m_configZoomedThumbnail;
 
     void readGeneralSettings ();
     void readThumbnailSettings ();
@@ -131,9 +134,9 @@ public:
 
 private:
     kpViewScrollableContainer *m_scrollView;
-    kpView *m_mainView;
+    kpZoomedView *m_mainView;
     kpThumbnail *m_thumbnail;
-    kpView *m_thumbnailView;
+    kpThumbnailView *m_thumbnailView;
     kpDocument *m_document;
     kpViewManager *m_viewManager;
     kpColorToolBar *m_colorToolBar;
@@ -163,12 +166,6 @@ private slots:
 private:
     virtual void moveEvent (QMoveEvent *e);
 
-public:
-    void drawTransparentBackground (QPainter *painter,
-                                    int viewWidth, int viewHeight,
-                                    const QRect &rect,
-                                    bool isPreview = false);
-
 private slots:
     void slotUpdateCaption ();
     void slotDocumentRestored ();
@@ -343,7 +340,7 @@ private slots:
 
 private:
     void sendFilenameToPrinter (KPrinter *printer);
-    void sendPixmapToPrinter (KPrinter *printer);
+    void sendPixmapToPrinter (KPrinter *printer, bool showPrinterSetupDialog);
 
 private slots:
     void slotPrint ();
@@ -385,6 +382,9 @@ private:
     kpDocumentSaveOptions m_lastCopyToSaveOptions;
     bool m_copyToFirstTime;
 
+public:
+    QPopupMenu *selectionToolRMBMenu ();
+
 private slots:
     void slotCut ();
     void slotCopy ();
@@ -439,7 +439,8 @@ private:
             *m_actionFitToPage, *m_actionFitToWidth, *m_actionFitToHeight,
             *m_actionZoomIn, *m_actionZoomOut;
     KSelectAction *m_actionZoom;
-    KToggleAction *m_actionShowGrid, *m_actionShowThumbnail;
+    KToggleAction *m_actionShowGrid,
+                  *m_actionShowThumbnail, *m_actionZoomedThumbnail;
 
     QValueVector <int> m_zoomList;
 
@@ -447,7 +448,7 @@ private:
     void sendZoomListToActionZoom ();
     int zoomLevelFromString (const QString &string);
     QString zoomLevelToString (int zoomLevel);
-    void zoomTo (int zoomLevel);
+    void zoomTo (int zoomLevel, bool centerUnderCursor = false);
 
 private slots:
     void finishZoomTo ();
@@ -458,9 +459,18 @@ private slots:
     void slotFitToWidth ();
     void slotFitToHeight ();
 
+public:
+    void zoomIn (bool centerUnderCursor = false);
+    void zoomOut (bool centerUnderCursor = false);
+    
+public slots:
     void slotZoomIn ();
     void slotZoomOut ();
 
+private:
+    void zoomAccordingToZoomAction (bool centerUnderCursor = false);
+    
+private slots:
     void slotZoom ();
 
     void slotShowGridToggled ();
@@ -486,7 +496,16 @@ public:
 private slots:
     void slotSaveThumbnailGeometry ();
     void slotShowThumbnailToggled ();
+    void updateThumbnailZoomed ();
+    void slotZoomedThumbnailToggled ();
+    void slotThumbnailShowRectangleToggled ();
+
 private:
+    void enableViewZoomedThumbnail (bool enable = true);
+    void enableViewShowThumbnailRectangle (bool enable = true);
+    void enableThumbnailOptionActions (bool enable = true);
+    void createThumbnailView ();
+    void destroyThumbnailView ();
     void updateThumbnail ();
 
 
@@ -640,6 +659,19 @@ private:
     QString m_textOldFontFamily;
     int m_textOldFontSize;
 
+
+    /*
+     * Help Menu
+     */
+private:
+    void setupHelpMenuActions ();
+    void enableHelpMenuDocumentActions (bool enable = true);
+
+private slots:
+    void slotHelpTakingScreenshots ();
+    void slotHelpTakingScreenshotsFollowLink (const QString &link);
+
+
 private:
     // There is no need to maintain binary compatibility at this stage.
     // The d-pointer is just so that you can experiment without recompiling
@@ -647,9 +679,5 @@ private:
     class kpMainWindowPrivate *d;
 };
 
-struct kpMainWindowPrivate
-{
-};
-
 
 #endif  // KP_MAIN_WINDOW_H
