@@ -12,7 +12,7 @@
  *   Copyright (C) 2004 by Christoph Cullmann <crossfire@babylon2k.de>     *
  *   Copyright (C) 2004 by Henrique Pinto <stampede@coltec.ufmg.br>        *
  *   Copyright (C) 2004 by Waldo Bastian <bastian@kde.org>                 *
- *   Copyright (C) 2004 by Albert Astals Cid <tsdgeos@terra.es>            *
+ *   Copyright (C) 2004-2006 by Albert Astals Cid <tsdgeos@terra.es>       *
  *   Copyright (C) 2004 by Antti Markus <antti.markus@starman.ee>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -132,7 +132,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	m_splitter->setOpaqueResize( true );
 	setWidget( m_splitter );
 	
-	m_showLeftPanel = new KToggleAction( i18n( "Show &Navigation Panel"), 0, this, SLOT( slotShowLeftPanel() ), actionCollection(), "show_leftpanel" );
+	m_showLeftPanel = new KToggleAction( i18n( "Show &Navigation Panel"), "show_side_panel", 0, this, SLOT( slotShowLeftPanel() ), actionCollection(), "show_leftpanel" );
 	m_showLeftPanel->setCheckedState( i18n( "Hide &Navigation Panel") );
 	m_showLeftPanel->setShortcut( "CTRL+L" );
 	m_showLeftPanel->setChecked( KpdfSettings::showLeftPanel() );
@@ -149,9 +149,9 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 
 	int index;
 	// [left toolbox: Table of Contents] | []
-	TOC * tocFrame = new TOC( m_toolBox, m_document );
-	connect(tocFrame, SIGNAL(hasTOC(bool)), this, SLOT(enableTOC(bool)));
-	index = m_toolBox->addItem( tocFrame, QIconSet(SmallIcon("text_left")), i18n("Contents") );
+	m_tocFrame = new TOC( m_toolBox, m_document );
+	connect(m_tocFrame, SIGNAL(hasTOC(bool)), this, SLOT(enableTOC(bool)));
+	index = m_toolBox->addItem( m_tocFrame, QIconSet(SmallIcon("text_left")), i18n("Contents") );
 	m_toolBox->setItemToolTip(index, i18n("Contents"));
 	enableTOC( false );
 
@@ -186,8 +186,8 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	miniSpacer->setFixedHeight( 6 );
 	miniBarLayout->addWidget( miniSpacer );
 	// widgets: [../[../MiniBar]] | []
-	MiniBar * miniBar = new MiniBar( miniBarContainer, m_document );
-	miniBarLayout->addWidget( miniBar );
+	m_miniBar = new MiniBar( miniBarContainer, m_document );
+	miniBarLayout->addWidget( m_miniBar );
 
 	// widgets: [] | [right 'pageView']
 	m_pageView = new PageView( m_splitter, m_document );
@@ -199,8 +199,8 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	m_document->addObserver( this );
 	m_document->addObserver( m_thumbnailList );
 	m_document->addObserver( m_pageView );
-	m_document->addObserver( tocFrame );
-	m_document->addObserver( miniBar );
+	m_document->addObserver( m_tocFrame );
+	m_document->addObserver( m_miniBar );
 
 	// ACTIONS
 	KActionCollection * ac = actionCollection();
@@ -209,19 +209,19 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	m_gotoPage = KStdAction::gotoPage( this, SLOT( slotGoToPage() ), ac, "goto_page" );
 	m_gotoPage->setShortcut( "CTRL+G" );
 	// dirty way to activate gotopage when pressing miniBar's button
-	connect( miniBar, SIGNAL( gotoPage() ), m_gotoPage, SLOT( activate() ) );
+	connect( m_miniBar, SIGNAL( gotoPage() ), m_gotoPage, SLOT( activate() ) );
 
 	m_prevPage = KStdAction::prior(this, SLOT(slotPreviousPage()), ac, "previous_page");
 	m_prevPage->setWhatsThis( i18n( "Moves to the previous page of the document" ) );
-	m_prevPage->setShortcut( "Backspace" );
+	m_prevPage->setShortcut( 0 );
 	// dirty way to activate prev page when pressing miniBar's button
-	connect( miniBar, SIGNAL( prevPage() ), m_prevPage, SLOT( activate() ) );
+	connect( m_miniBar, SIGNAL( prevPage() ), m_prevPage, SLOT( activate() ) );
 
 	m_nextPage = KStdAction::next(this, SLOT(slotNextPage()), ac, "next_page" );
 	m_nextPage->setWhatsThis( i18n( "Moves to the next page of the document" ) );
-	m_nextPage->setShortcut( "Space" );
+	m_nextPage->setShortcut( 0 );
 	// dirty way to activate next page when pressing miniBar's button
-	connect( miniBar, SIGNAL( nextPage() ), m_nextPage, SLOT( activate() ) );
+	connect( m_miniBar, SIGNAL( nextPage() ), m_nextPage, SLOT( activate() ) );
 
 	m_firstPage = KStdAction::firstPage( this, SLOT( slotGotoFirst() ), ac, "first_page" );
 	m_firstPage->setWhatsThis( i18n( "Moves to the first page of the document" ) );
@@ -274,6 +274,8 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	connect( m_watcher, SIGNAL( dirty( const QString& ) ), this, SLOT( slotFileDirty( const QString& ) ) );
 	m_dirtyHandler = new QTimer( this );
 	connect( m_dirtyHandler, SIGNAL( timeout() ),this, SLOT( slotDoFileDirty() ) );
+	m_saveSplitterSizeTimer = new QTimer( this );
+	connect( m_saveSplitterSizeTimer, SIGNAL( timeout() ),this, SLOT( saveSplitterSize() ) );
 
 	slotNewConfig();
 
@@ -289,6 +291,11 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 
 Part::~Part()
 {
+    delete m_tocFrame;
+    delete m_pageView;
+    delete m_thumbnailList;
+    delete m_miniBar;
+
     delete m_document;
     if ( --m_count == 0 )
         delete globalParams;
@@ -360,20 +367,25 @@ bool Part::openFile()
         QString app = KStandardDirs::findExe( "ps2pdf" );
         if ( !app.isNull() )
         {
-            KTempFile tf( QString::null, ".pdf" );
-            if ( tf.status() == 0 )
-            {
-                tf.close();
-                m_temporaryLocalFile = tf.name();
+            if ( QFile::exists(m_file) )
+	    {
+                KTempFile tf( QString::null, ".pdf" );
+                if ( tf.status() == 0 )
+                {
+                    tf.close();
+                    m_temporaryLocalFile = tf.name();
 
-                KProcess *p = new KProcess;
-                *p << app;
-                *p << m_file << m_temporaryLocalFile;
-                m_pageView->showText(i18n("Converting from ps to pdf..."), 0);
-                connect(p, SIGNAL(processExited(KProcess *)), this, SLOT(psTransformEnded()));
-                p -> start();
-                return true;
+                    KProcess *p = new KProcess;
+                    *p << app;
+                    *p << m_file << m_temporaryLocalFile;
+                    m_pageView->showText(i18n("Converting from ps to pdf..."), 0);
+                    connect(p, SIGNAL(processExited(KProcess *)), this, SLOT(psTransformEnded()));
+                    p -> start();
+                    return true;
+                }
+                else return false;
             }
+            else return false;
         }
         else
         {
@@ -438,6 +450,7 @@ bool Part::openURL(const KURL &url)
         KMessageBox::error( widget(), i18n("Could not open %1").arg( url.prettyURL() ) );
     else
         m_viewportDirty.pageNumber = -1;
+    emit enablePrintAction(b);
     return b;
 }
 
@@ -478,7 +491,7 @@ bool Part::eventFilter( QObject * watched, QEvent * e )
 {
     // if pageView has been resized, save splitter sizes
     if ( watched == m_pageView && e->type() == QEvent::Resize )
-        saveSplitterSize();
+        m_saveSplitterSizeTimer->start(500, true);
 
     // only intercept events, don't block them
     return false;
@@ -573,8 +586,13 @@ void Part::enableTOC(bool enable)
 
 void Part::psTransformEnded()
 {
+	QString aux = m_file;
 	m_file = m_temporaryLocalFile;
 	openFile();
+	m_file = aux; // so watching works, we have to watch the ps file not the autogenerated pdf
+	m_watcher->removeFile(m_temporaryLocalFile);
+	if ( !m_watcher->contains(m_file) )
+		m_watcher->addFile(m_file);
 }
 
 void Part::cannotQuit()
@@ -627,7 +645,7 @@ void Part::slotGoToPage()
 
 void Part::slotPreviousPage()
 {
-    if ( m_document->isOpened() && !m_document->currentPage() < 1 )
+    if ( m_document->isOpened() && !(m_document->currentPage() < 1) )
         m_document->setViewportPage( m_document->currentPage() - 1 );
 }
 
@@ -663,6 +681,8 @@ void Part::slotFind()
 {
     KFindDialog dlg( widget() );
     dlg.setHasCursor( false );
+    if ( !m_searchHistory.empty() )
+        dlg.setFindHistory( m_searchHistory );
 #if KDE_IS_VERSION(3,3,90)
     dlg.setSupportsBackwardsFind( false );
     dlg.setSupportsWholeWordsFind( false );
@@ -670,6 +690,7 @@ void Part::slotFind()
 #endif
     if ( dlg.exec() == QDialog::Accepted )
     {
+        m_searchHistory = dlg.findHistory();
         m_searchStarted = true;
         m_document->resetSearch( PART_SEARCH_ID );
         m_document->searchText( PART_SEARCH_ID, dlg.pattern(), false, dlg.options() & KFindDialog::CaseSensitive,
@@ -679,9 +700,7 @@ void Part::slotFind()
 
 void Part::slotFindNext()
 {
-    if ( m_searchStarted )
-        m_document->continueSearch( PART_SEARCH_ID );
-    else
+    if (!m_document->continueLastSearch())
         slotFind();
 }
 
@@ -690,6 +709,11 @@ void Part::slotSaveFileAs()
     KURL saveURL = KFileDialog::getSaveURL( url().isLocalFile() ? url().url() : url().fileName(), QString::null, widget() );
     if ( saveURL.isValid() && !saveURL.isEmpty() )
     {
+        if (saveURL == url())
+        {
+            KMessageBox::information( widget(), i18n("You are trying to overwrite \"%1\" with itself. This is not allowed. Please save it in another location.").arg(saveURL.filename()) );
+            return;
+        }
         if ( KIO::NetAccess::exists( saveURL, false, widget() ) )
         {
             if (KMessageBox::warningContinueCancel( widget(), i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?").arg(saveURL.filename()), QString::null, i18n("Overwrite")) != KMessageBox::Continue)
@@ -931,15 +955,21 @@ void Part::doPrint(KPrinter &printer)
     }
 }
 
-void Part::restoreDocument(const KURL &url, int page)
+void Part::restoreDocument(KConfig* config)
 {
-  if (openURL(url)) goToPage(page);
+  KURL url ( config->readPathEntry( "URL" ) );
+  if ( url.isValid() )
+  {
+    QString viewport = config->readEntry( "Viewport" );
+    if (!viewport.isEmpty()) m_document->setNextDocumentViewport( DocumentViewport( viewport ) );
+    openURL( url );
+  }
 }
 
 void Part::saveDocumentRestoreInfo(KConfig* config)
 {
   config->writePathEntry( "URL", url().url() );
-  if (m_document->pages() > 0) config->writeEntry( "Page", m_document->currentPage() + 1 );
+  config->writeEntry( "Viewport", m_document->viewport().toString() );
 }
 
 /*
