@@ -5,7 +5,7 @@
  *   Copyright (C) 2003 by Andy Goossens <andygoossens@telenet.be>         *
  *   Copyright (C) 2003 by Laurent Montel <montel@kde.org>                 *
  *   Copyright (C) 2004 by Dominique Devriese <devriese@kde.org>           *
- *   Copyright (C) 2004 by Albert Astals Cid <tsdgeos@terra.es>            *
+ *   Copyright (C) 2004-2006 by Albert Astals Cid <tsdgeos@terra.es>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -41,6 +41,8 @@ class ThumbnailController;
 class PageView;
 class PresentationWidget;
 class SearchWidget;
+class TOC;
+class MiniBar;
 
 namespace KPDF {
 
@@ -122,7 +124,7 @@ protected slots:
 public slots:
 	// connected to Shell action (and browserExtension), not local one
 	void slotPrint();
-	void restoreDocument(const KURL &url, int page);
+	void restoreDocument(KConfig* config);
 	void saveDocumentRestoreInfo(KConfig* config);
 	void slotFileDirty( const QString& );
 	void slotDoFileDirty();
@@ -139,16 +141,26 @@ private:
 	QWidget *m_leftPanel;
 	QToolBox *m_toolBox;
 	SearchWidget *m_searchWidget;
-	ThumbnailList *m_thumbnailList;
-	PageView *m_pageView;
+	QGuardedPtr<ThumbnailList> m_thumbnailList;
+	QGuardedPtr<PageView> m_pageView;
+	QGuardedPtr<TOC> m_tocFrame;
+	QGuardedPtr<MiniBar> m_miniBar;
 	QGuardedPtr<PresentationWidget> m_presentationWidget;
 
 	// static instances counter
 	static unsigned int m_count;
 
+	// this is a hack because we can not use writeConfig on part destructor
+	// and we don't want to writeconfig every time someone moves the splitter
+	// so we use a QTimer each 500 ms
+	QTimer *m_saveSplitterSizeTimer;
+
 	KDirWatch *m_watcher;
 	QTimer *m_dirtyHandler;
 	DocumentViewport m_viewportDirty;
+	
+	// Remember the search history
+	QStringList m_searchHistory;
 
 	// actions
 	KAction *m_gotoPage;
