@@ -25,6 +25,7 @@
 #include <qapplication.h>
 #include <qclipboard.h>
 #include <dcopclient.h>
+#include <kcursor.h>
 #include <kiconloader.h>
 #include <kurldrag.h>
 #include <kaction.h>
@@ -158,15 +159,16 @@ PageView::PageView( QWidget *parent, KPDFDocument *document )
     // conntect the padding of the viewport to pixmaps requests
     connect( this, SIGNAL(contentsMoving(int, int)), this, SLOT(slotRequestVisiblePixmaps(int, int)) );
 
-    // show initial welcome text
-    d->messageWindow->display( i18n( "Welcome" ), PageViewMessage::Info, 2000 );
-
     // set a corner button to resize the view to the page size
 //    QPushButton * resizeButton = new QPushButton( viewport() );
 //    resizeButton->setPixmap( SmallIcon("crop") );
 //    setCornerWidget( resizeButton );
 //    resizeButton->setEnabled( false );
     // connect(...);
+    setInputMethodEnabled( true );
+
+    // schedule the welcome message
+    QTimer::singleShot( 0, this, SLOT( slotShowWelcome() ) );
 }
 
 PageView::~PageView()
@@ -624,12 +626,7 @@ void PageView::keyPressEvent( QKeyEvent * e )
         else if( !e->text().isEmpty() )
         {
             d->typeAheadString += e->text();
-            bool found = d->document->searchText( PAGEVIEW_SEARCH_ID, d->typeAheadString, false, false,
-                    KPDFDocument::NextMatch, true, qRgb( 128, 255, 128 ), true );
-            QString status = found ? i18n("Text found: \"%1\".") : i18n("Text not found: \"%1\".");
-            d->messageWindow->display( status.arg(d->typeAheadString.lower()),
-                                       found ? PageViewMessage::Find : PageViewMessage::Warning, 4000 );
-            d->findTimeoutTimer->start( 3000, true );
+            doTypeAheadSearch();
         }
         return;
     }
@@ -668,6 +665,7 @@ void PageView::keyPressEvent( QKeyEvent * e )
     {
         case Key_Up:
         case Key_PageUp:
+	case Key_Backspace:
             // if in single page mode and at the top of the screen, go to previous page
             if ( KpdfSettings::viewContinuous() || verticalScrollBar()->value() > verticalScrollBar()->minValue() )
             {
@@ -688,6 +686,7 @@ void PageView::keyPressEvent( QKeyEvent * e )
             break;
         case Key_Down:
         case Key_PageDown:
+	case Key_Space:
             // if in single page mode and at the bottom of the screen, go to next page
             if ( KpdfSettings::viewContinuous() || verticalScrollBar()->value() < verticalScrollBar()->maxValue() )
             {
@@ -722,6 +721,7 @@ void PageView::keyPressEvent( QKeyEvent * e )
                     slotAutoScoll();
                 return;
             }
+            // else fall trhough
         default:
             e->ignore();
             return;
@@ -734,6 +734,19 @@ void PageView::keyPressEvent( QKeyEvent * e )
     }
 }
 
+void PageView::imEndEvent( QIMEvent * e )
+{
+    if( d->typeAheadActive )
+    {
+        if( !e->text().isEmpty() )
+        {
+            d->typeAheadString += e->text();
+            doTypeAheadSearch();
+            e->accept();
+        }
+    }
+}
+
 void PageView::contentsMouseMoveEvent( QMouseEvent * e )
 {
     // don't perform any mouse action when no document is shown
@@ -745,7 +758,7 @@ void PageView::contentsMouseMoveEvent( QMouseEvent * e )
         return;
 
     // if holding mouse mid button, perform zoom
-    if ( (e->state() & MidButton) && d->mouseMidStartY > 0 )
+    if ( (e->state() & MidButton) && d->mouseMidStartY >= 0 )
     {
         int deltaY = d->mouseMidStartY - e->globalPos().y();
         d->mouseMidStartY = e->globalPos().y();
@@ -828,7 +841,7 @@ void PageView::contentsMousePressEvent( QMouseEvent * e )
     if ( e->button() & MidButton )
     {
         d->mouseMidStartY = e->globalPos().y();
-        setCursor( sizeVerCursor );
+        setCursor( KCursor::sizeVerCursor() );
         return;
     }
 
@@ -845,7 +858,7 @@ void PageView::contentsMousePressEvent( QMouseEvent * e )
             {
                 d->mouseGrabPos = d->mouseOnRect ? QPoint() : d->mousePressPos;
                 if ( !d->mouseOnRect )
-                    setCursor( sizeAllCursor );
+                    setCursor( KCursor::sizeAllCursor() );
             }
             break;
 
@@ -1507,19 +1520,29 @@ void PageView::updateCursor( const QPoint &p )
         // if over a ObjectRect (of type Link) change cursor to hand
         d->mouseOnRect = pageItem->page()->hasObject( ObjectRect::Link, nX, nY );
         if ( d->mouseOnRect )
-            setCursor( pointingHandCursor );
+            setCursor( KCursor::handCursor() );
         else
-            setCursor( arrowCursor );
+            setCursor( KCursor::arrowCursor() );
     }
     else
     {
         // if there's no page over the cursor and we were showing the pointingHandCursor
         // go back to the normal one
         d->mouseOnRect = false;
-        setCursor( arrowCursor );
+        setCursor( KCursor::arrowCursor() );
     }
 }
 
+void PageView::doTypeAheadSearch()
+{
+    bool found = d->document->searchText( PAGEVIEW_SEARCH_ID, d->typeAheadString, false, false,
+                                          KPDFDocument::NextMatch, true, qRgb( 128, 255, 128 ), true );
+    QString status = found ? i18n("Text found: \"%1\".") : i18n("Text not found: \"%1\".");
+    d->messageWindow->display( status.arg(d->typeAheadString.lower()),
+                               found ? PageViewMessage::Find : PageViewMessage::Warning, 4000 );
+    d->findTimeoutTimer->start( 3000, true );
+}
+
 //BEGIN private SLOTS
 void PageView::slotRelayoutPages()
 // called by: notifySetup, viewportResizeEvent, slotTwoPagesToggled, slotContinuousToggled, updateZoom
@@ -1879,6 +1902,12 @@ void PageView::findAheadStop()
     releaseKeyboard();
 }
 
+void PageView::slotShowWelcome()
+{
+    // show initial welcome text
+    d->messageWindow->display( i18n( "Welcome" ), PageViewMessage::Info, 2000 );
+}
+
 void PageView::slotZoom()
 {
     setFocus();
