@@ -641,7 +641,7 @@ void PMShell::slotControlPointMsg( const QString& msg )
 
 bool PMShell::overwriteURL( const KURL& u )
 {
-   int query = KMessageBox::Yes;
+   int query = KMessageBox::Continue;
 
    if( u.isLocalFile( ) )
    {
