@@ -22,6 +22,7 @@
 #include "pmvector.h"
 #include "pmdebug.h"
 
+inline const char* dcopTypeName( const PMVariant& ) { return "PMVariant"; }
 
 PMVariant::PMVariant( )
 {
@@ -89,7 +90,7 @@ PMVariant::PMVariant( const PMVariant& v )
 {
    m_pData = 0;
    m_dataType = PMVariant::None;
-   
+
    switch( v.m_dataType )
    {
       case PMVariant::Integer:
@@ -201,7 +202,7 @@ void PMVariant::clear( )
       case PMVariant::None:
          break;
    }
-   
+
    m_dataType = PMVariant::None;
    m_pData = 0;
 }
@@ -311,7 +312,7 @@ void PMVariant::setObject( PMObject* obj )
       m_dataType = PMVariant::ObjectPointer;
    }
    else
-      ( ( PMObject* ) m_pData ) = obj;
+      m_pData = obj;
 }
 
 int PMVariant::intData( ) const
@@ -389,13 +390,13 @@ PMObject* PMVariant::objectData( ) const
 bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
 {
    bool success = true;
-   
+
    switch( m_dataType )
    {
       case PMVariant::Integer:
       {
          int data = *( ( int* ) m_pData );
-         
+
          switch( t )
          {
             case PMVariant::Integer:
@@ -434,11 +435,11 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-      
+
       case PMVariant::Unsigned:
       {
          unsigned data = *( ( unsigned* ) m_pData );
-         
+
          switch( t )
          {
             case PMVariant::Integer:
@@ -477,11 +478,11 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-      
+
       case PMVariant::Double:
       {
          double data = *( ( double* ) m_pData );
-         
+
          switch( t )
          {
             case PMVariant::Integer:
@@ -520,11 +521,11 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-      
+
       case PMVariant::Bool:
       {
          bool data = *( ( bool* ) m_pData );
-         
+
          switch( t )
          {
             case PMVariant::Integer:
@@ -565,11 +566,11 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-      
+
       case PMVariant::ThreeState:
       {
          PMThreeState data = *( ( PMThreeState* ) m_pData );
-         
+
          switch( t )
          {
             case PMVariant::Integer:
@@ -614,11 +615,11 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-      
+
       case PMVariant::String:
       {
          QString data = *( ( QString* ) m_pData );
-         
+
          switch( t )
          {
             case PMVariant::Integer:
@@ -677,7 +678,7 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-      
+
       case PMVariant::Vector:
       {
          switch( t )
@@ -699,7 +700,7 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-         
+
       case PMVariant::Color:
       {
          switch( t )
@@ -724,7 +725,7 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
          }
          break;
       }
-      
+
       case PMVariant::ObjectPointer:
          success = ( t == PMVariant::ObjectPointer );
          break;
@@ -736,3 +737,184 @@ bool PMVariant::convertTo( PMVariant::PMVariantDataType t )
 
    return success;
 }
+
+QString PMVariant::asString( ) const
+{
+   QString tmp;
+
+   switch( m_dataType )
+   {
+      case PMVariant::Integer:
+      {
+         int data = *( ( int* ) m_pData );
+
+         tmp.setNum( data );
+         break;
+      }
+      case PMVariant::Unsigned:
+      {
+         unsigned data = *( ( unsigned* ) m_pData );
+
+         tmp.setNum( data );
+         break;
+      }
+      case PMVariant::Double:
+      {
+         double data = *( ( double* ) m_pData );
+
+         tmp.setNum( data );
+         break;
+      }
+      case PMVariant::Bool:
+      {
+         bool data = *( ( bool* ) m_pData );
+
+         if( data )
+            tmp = "true";
+         else
+            tmp = "false";
+         break;
+      }
+      case PMVariant::ThreeState:
+      {
+         PMThreeState data = *( ( PMThreeState* ) m_pData );
+
+         if( data == PMTrue )
+            tmp = "true";
+         else if( data == PMFalse )
+            tmp = "false";
+         else
+            tmp = "unspecified";
+         break;
+      }
+      case PMVariant::String:
+      {
+         tmp = *( ( QString* ) m_pData );
+         break;
+      }
+      case PMVariant::Vector:
+      {
+         PMVector v = *( ( PMVector* ) m_pData );
+         tmp = v.serializeXML( );
+         break;
+      }
+
+      case PMVariant::Color:
+      {
+         PMColor c = *( ( PMColor* ) m_pData );
+         tmp = c.serializeXML( );
+         break;
+      }
+
+      case PMVariant::ObjectPointer:
+         tmp = "<object_pointer>";
+         break;
+
+      case PMVariant::None:
+         tmp = "<none>"; 
+         break;
+
+      default:
+         tmp = "<unknown>";
+         break;
+   }
+
+   return tmp;
+}
+
+bool PMVariant::fromString( const PMVariant::PMVariantDataType t, const QString& value )
+{
+   bool success;
+
+   switch( t )
+   {
+      case PMVariant::Integer:
+      { 
+         int i = value.toInt( &success );
+         if( success )
+            setInt( i );
+         break;
+      }
+      case PMVariant::Unsigned:
+      {
+         unsigned u = value.toUInt( &success );
+         if( success )
+            setUnsigned( u );
+         break;
+      }
+      case PMVariant::Double:
+      {
+         double d = value.toDouble( &success );
+         if( success )
+            setDouble( d );
+         break;
+      }
+      case PMVariant::Bool:
+         success = true;
+         if( value == "true" || value == "on" || value == "yes" )
+            setBool( true );
+         else if( value == "false" || value == "off" || value == "no" )
+            setBool( false );
+         else
+            success = false;
+         break;
+      case PMVariant::ThreeState:
+         success = true; // Assume success, set to false if we fail.
+         if( value == "true" || value == "on" || value == "yes" )
+            setThreeState( PMTrue );
+         else if( value == "false" || value == "off" || value == "no" )
+            setThreeState( PMFalse );
+         else if( value == "unspecified" )
+            setThreeState( PMUnspecified );
+         else
+            success = false;
+         break;
+      case PMVariant::String:
+         setString( value );
+         success = true;
+         break;
+      case PMVariant::Vector:
+         {
+            PMVector v;
+            v.loadXML( value );
+            setVector( v );
+            success = true;
+            break;
+         }
+      case PMVariant::Color:
+         {
+            PMColor c;
+            c.loadXML( value );
+            setColor( c );
+            success = true;
+            break;
+         }
+      default:
+         success = false;
+   }
+   return success;
+}
+
+QDataStream& operator<<( QDataStream& stream, const PMVariant& value )
+{
+   stream << (Q_INT8)value.dataType( );
+   stream << value.asString( );
+
+   return stream;
+}
+
+QDataStream& operator>>( QDataStream& stream, PMVariant& value ) 
+{
+   Q_INT8 type;
+   PMVariant::PMVariantDataType dataType;
+   QString str;
+
+   stream >> type;
+   dataType = (PMVariant::PMVariantDataType)type;
+
+   stream >> str;
+
+   value.fromString(dataType, str);
+
+   return stream;
+}
