@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 1998-2003 Carsten Pfeiffer <pfeiffer@kde.org>
+   Copyright (C) 1998-2006 Carsten Pfeiffer <pfeiffer@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <stdio.h>
@@ -46,6 +46,7 @@
 #include <kmenubar.h>
 #include <kmessagebox.h>
 #include <kpopupmenu.h>
+#include <kprotocolinfo.h>
 #include <kpropertiesdialog.h>
 #include <kprotocolinfo.h>
 #include <kstatusbar.h>
@@ -59,13 +60,16 @@
 #include <kwin.h>
 #include <kstdguiitem.h>
 
+#include <kdebug.h>
+
 #include "aboutwidget.h"
 #include "filewidget.h"
+#include "filecache.h"
 #include "imdata.h"
 #include "imagewindow.h"
 #include "imlibwidget.h"
 #include "kuick.h"
-#include "kuickio.h"
+#include "kuickfile.h"
 
 #ifdef index
 #undef index
@@ -96,7 +100,8 @@ KuickShow::KuickShow( const char *name )
       m_viewer( 0L ),
       oneWindowAction( 0L ),
       m_accel( 0L ),
-      m_delayedRepeatItem( 0L )
+      m_delayedRepeatItem( 0L ),
+      m_slideShowStopped(false)
 {
     aboutWidget = 0L;
     kdata = new KuickData;
@@ -120,7 +125,21 @@ KuickShow::KuickShow( const char *name )
     // either a directory to display, an absolute path, a relative path, or a URL
     KURL startDir;
     startDir.setPath( QDir::currentDirPath() + '/' );
-    for ( int i = 0; i < args->count(); i++ ) {
+
+    int numArgs = args->count();
+    if ( numArgs >= 10 )
+    {
+        if ( KMessageBox::warningYesNo(
+                 this,
+                 i18n("Do you really want to display these %1 images at the same time? This might be quite resource intensive and could overload your computer.\nIf you choose %2, only the first image will be shown.").arg(numArgs).arg(KStdGuiItem::no().plainText()),
+                 i18n("Display %1 images?").arg(numArgs))
+             != KMessageBox::Yes)
+        {
+            numArgs = 1;
+        }
+    }
+
+    for ( int i = 0; i < numArgs; i++ ) {
         KURL url = args->url( i );
         KFileItem item( KFileItem::Unknown, KFileItem::Unknown, url, false );
 
@@ -144,12 +163,13 @@ KuickShow::KuickShow( const char *name )
         {
             KMimeType::Ptr mime = KMimeType::findByURL( url );
             QString name = mime->name();
-#if KDE_VERSION >= 310
             if ( name == "application/octet-stream" ) // unknown -> stat()
                 name = KIO::NetAccess::mimetype( url, this );
-#endif
 
-            if ( name.startsWith( "image/" ) )
+	    // text/* is a hack for bugs.kde.org-attached-images urls.
+	    // The real problem here is that NetAccess::mimetype does a HTTP HEAD, which doesn't
+	    // always return the right mimetype. The rest of KDE start a get() instead....
+            if ( name.startsWith( "image/" ) || name.startsWith( "text/" ) )
             {
                 FileWidget::setImage( item, true );
                 showImage( &item, true, false, true );
@@ -170,7 +190,8 @@ KuickShow::KuickShow( const char *name )
 
     if ( s_viewers.isEmpty() || isDir ) {
         initGUI( startDir );
-        show();
+	if (!kapp->isRestored()) // during session management, readProperties() will show()
+            show();
     }
 
     else { // don't show browser, when image on commandline
@@ -187,6 +208,7 @@ KuickShow::~KuickShow()
     if ( m_viewer )
         m_viewer->close( true );
 
+    FileCache::shutdown();
     free( id );
     kapp->quit();
 
@@ -196,11 +218,17 @@ KuickShow::~KuickShow()
 // TODO convert to use xmlui file
 void KuickShow::initGUI( const KURL& startDir )
 {
-    fileWidget = new FileWidget( startDir, this, "MainWidget" );
+	KURL startURL( startDir );
+	if ( !KProtocolInfo::supportsListing( startURL ) )
+		startURL = KURL();
+
+    fileWidget = new FileWidget( startURL, this, "MainWidget" );
     setFocusProxy( fileWidget );
 
     KActionCollection *coll = fileWidget->actionCollection();
 
+    redirectDeleteAndTrashActions(coll);
+
     connect( fileWidget, SIGNAL( fileSelected( const KFileItem * ) ),
              this, SLOT( slotSelected( const KFileItem * ) ));
 
@@ -252,6 +280,9 @@ void KuickShow::initGUI( const KURL& startDir )
                                        KShortcut(),
                                        this, SLOT( slotShowInSameWindow() ),
                                        coll, "kuick_showInSameWindow" );
+    KAction *showFullscreen = new KAction( i18n("Show Image in Fullscreen Mode"),
+					   KShortcut(), this, SLOT( slotShowFullscreen() ),
+					   coll, "kuick_showFullscreen" );
 
     KAction *quit = KStdAction::quit( this, SLOT(slotQuit()), coll, "quit");
 
@@ -265,6 +296,7 @@ void KuickShow::initGUI( const KURL& startDir )
     open->plug( fileMenu );
     showInOther->plug( fileMenu );
     showInSame->plug( fileMenu );
+    showFullscreen->plug( fileMenu );
     fileMenu->insertSeparator();
     slide->plug( fileMenu );
     print->plug( fileMenu );
@@ -371,7 +403,6 @@ void KuickShow::initGUI( const KURL& startDir )
     dirSelected( fileWidget->url() );
 
     setCentralWidget( fileWidget );
-
     setupGUI( KMainWindow::Save );
 
     coll->action( "reload" )->setShortcut( KStdAccel::reload() );
@@ -383,6 +414,23 @@ void KuickShow::initGUI( const KURL& startDir )
     coll->action( "separate dirs" )->setShortcut(Key_F12);
 }
 
+void KuickShow::redirectDeleteAndTrashActions(KActionCollection *coll)
+{
+    KAction *action = coll->action("delete");
+    if (action)
+    {
+        action->disconnect(fileWidget);
+        connect(action, SIGNAL(activated()), this, SLOT(slotDeleteCurrentImage()));
+    }
+
+    action = coll->action("trash");
+    if (action)
+    {
+        action->disconnect(fileWidget);
+        connect(action, SIGNAL(activated()), this, SLOT(slotTrashCurrentImage()));
+    }
+}
+
 void KuickShow::slotSetURL( const KURL& url )
 {
     fileWidget->setURL( url, true );
@@ -403,7 +451,7 @@ void KuickShow::viewerDeleted()
 
     if ( !haveBrowser() && s_viewers.isEmpty() ) {
         saveSettings();
-
+        FileCache::shutdown();
         ::exit(0);
     }
 
@@ -449,6 +497,7 @@ void KuickShow::slotHighlighted( const KFileItem *fi )
     fileWidget->actionCollection()->action("kuick_print")->setEnabled( image );
     fileWidget->actionCollection()->action("kuick_showInSameWindow")->setEnabled( image );
     fileWidget->actionCollection()->action("kuick_showInOtherWindow")->setEnabled( image );
+    fileWidget->actionCollection()->action("kuick_showFullscreen")->setEnabled( image );
 }
 
 void KuickShow::dirSelected( const KURL& url )
@@ -474,7 +523,7 @@ void KuickShow::showFileItem( ImageWindow * /*view*/,
 
 }
 
-void KuickShow::showImage( const KFileItem *fi,
+bool KuickShow::showImage( const KFileItem *fi,
                            bool newWindow, bool fullscreen, bool moveToTopLeft )
 {
     newWindow  |= !m_viewer;
@@ -483,15 +532,22 @@ void KuickShow::showImage( const KFileItem *fi,
 
         if ( newWindow ) {
             m_viewer = new ImageWindow( kdata->idata, id, 0L, "image window" );
+            m_viewer->setFullscreen( fullscreen );
             s_viewers.append( m_viewer );
 
             connect( m_viewer, SIGNAL( destroyed() ), SLOT( viewerDeleted() ));
             connect( m_viewer, SIGNAL( sigFocusWindow( ImageWindow *) ),
                      this, SLOT( slotSetActiveViewer( ImageWindow * ) ));
-            connect( m_viewer, SIGNAL( sigBadImage(const QString& ) ),
-                     this, SLOT( messageCantLoadImage(const QString &) ));
+            connect( m_viewer, SIGNAL( sigImageError(const KuickFile *, const QString& ) ),
+                     this, SLOT( messageCantLoadImage(const KuickFile *, const QString &) ));
             connect( m_viewer, SIGNAL( requestImage( ImageWindow *, int )),
                      this, SLOT( slotAdvanceImage( ImageWindow *, int )));
+	    connect( m_viewer, SIGNAL( pauseSlideShowSignal() ),
+		     this, SLOT( pauseSlideShow() ) );
+            connect( m_viewer, SIGNAL (deleteImage (ImageWindow *)),
+                     this, SLOT (slotDeleteCurrentImage (ImageWindow *)));
+            connect( m_viewer, SIGNAL (trashImage (ImageWindow *)),
+                     this, SLOT (slotTrashCurrentImage (ImageWindow *)));
             if ( s_viewers.count() == 1 && moveToTopLeft ) {
                 // we have to move to 0x0 before showing _and_
                 // after showing, otherwise we get some bogus geometry()
@@ -509,18 +565,19 @@ void KuickShow::showImage( const KFileItem *fi,
         // Imlib can't load).
         ImageWindow *safeViewer = m_viewer;
 
-        QString filename;
-        KIO::NetAccess::download(fi->url(), filename, this);
+//        file->waitForDownload( this );
+//        QString filename;
+//        KIO::NetAccess::download(fi->url(), filename, this);
 
-        if ( !safeViewer->showNextImage( filename ) ) {
+        if ( !safeViewer->showNextImage( fi->url() ) ) {
             m_viewer = safeViewer;
             safeViewer->close( true ); // couldn't load image, close window
         }
         else {
-            safeViewer->setFullscreen( fullscreen );
+//            safeViewer->setFullscreen( fullscreen );
 
             if ( newWindow ) {
-                safeViewer->show();
+//                safeViewer->show();
 
                 if ( !fullscreen && s_viewers.count() == 1 && moveToTopLeft ) {
                     // the WM might have moved us after showing -> strike back!
@@ -533,12 +590,114 @@ void KuickShow::showImage( const KFileItem *fi,
                 KFileItem *item = 0L;                 // don't move cursor
                 item = fileWidget->getItem( FileWidget::Next, true );
                 if ( item )
-                    safeViewer->cacheImage( item->url().path() ); // FIXME
+                    safeViewer->cacheImage( item->url() );
             }
 
             m_viewer = safeViewer;
+	    return true;
         } // m_viewer created successfully
     } // isImage
+
+    return false;
+}
+
+void KuickShow::slotDeleteCurrentImage()
+{
+    performDeleteCurrentImage(fileWidget);
+}
+
+void KuickShow::slotTrashCurrentImage()
+{
+    performTrashCurrentImage(fileWidget);
+}
+
+void KuickShow::slotDeleteCurrentImage(ImageWindow *viewer)
+{
+    if (!fileWidget) {
+        delayAction(new DelayedRepeatEvent(viewer, DelayedRepeatEvent::DeleteCurrentFile, 0L));
+        return;
+    }
+    performDeleteCurrentImage(viewer);
+}
+
+void KuickShow::slotTrashCurrentImage(ImageWindow *viewer)
+{
+    if (!fileWidget) {
+        delayAction(new DelayedRepeatEvent(viewer, DelayedRepeatEvent::TrashCurrentFile, 0L));
+        return;
+    }
+    performTrashCurrentImage(viewer);
+}
+
+void KuickShow::performDeleteCurrentImage(QWidget *parent)
+{
+    assert(fileWidget != 0L);
+
+    KFileItemList list;
+    KFileItem *item = fileWidget->getCurrentItem(false);
+    list.append (item);
+
+    if (KMessageBox::warningContinueCancel(
+            parent,
+            i18n("<qt>Do you really want to delete\n <b>'%1'</b>?</qt>").arg(item->url().pathOrURL()),
+            i18n("Delete File"),
+            KStdGuiItem::del(),
+            "Kuick_delete_current_image")
+        != KMessageBox::Continue)
+    {
+        return;
+    }
+
+    tryShowNextImage();
+    fileWidget->del(list, false, false);
+}
+
+void KuickShow::performTrashCurrentImage(QWidget *parent)
+{
+    assert(fileWidget != 0L);
+
+    KFileItemList list;
+    KFileItem *item = fileWidget->getCurrentItem(false);
+    list.append (item);
+
+    if (KMessageBox::warningContinueCancel(
+            parent,
+            i18n("<qt>Do you really want to trash\n <b>'%1'</b>?</qt>").arg(item->url().pathOrURL()),
+            i18n("Trash File"),
+            KGuiItem(i18n("to trash", "&Trash"),"edittrash"),
+            "Kuick_trash_current_image")
+        != KMessageBox::Continue)
+    {
+        return;
+    }
+
+    tryShowNextImage();
+    fileWidget->trash(list, parent, false, false);
+}
+
+void KuickShow::tryShowNextImage()
+{
+    // move to next file item even if we have no viewer
+    KFileItem *next = fileWidget->getNext(true);
+    if (!next)
+        next = fileWidget->getPrevious(true);
+
+    // ### why is this necessary at all? Why does KDirOperator suddenly re-read the
+    // entire directory after a file was deleted/trashed!? (KDirNotify is the reason)
+    if (!m_viewer)
+        return;
+
+    if (next)
+        showImage(next, false);
+    else
+    {
+        if (!haveBrowser())
+        {
+            // ### when simply calling toggleBrowser(), this main window is completely messed up
+            QTimer::singleShot(0, this, SLOT(toggleBrowser()));
+        }
+        m_viewer->deleteLater();
+    }
 }
 
 void KuickShow::startSlideShow()
@@ -552,7 +711,21 @@ void KuickShow::startSlideShow()
         fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( false );
         showImage( item, !oneWindowAction->isChecked(),
                    kdata->slideshowFullscreen );
-        m_slideTimer->start( kdata->slideDelay );
+	if(kdata->slideDelay)
+            m_slideTimer->start( kdata->slideDelay );
+    }
+}
+
+void KuickShow::pauseSlideShow()
+{
+    if(m_slideShowStopped) {
+	if(kdata->slideDelay)
+	    m_slideTimer->start( kdata->slideDelay );
+	m_slideShowStopped = false;
+    }
+    else {
+	m_slideTimer->stop();
+	m_slideShowStopped = true;
     }
 }
 
@@ -586,8 +759,9 @@ void KuickShow::nextSlide()
 
 void KuickShow::nextSlide( KFileItem *item )
 {
-    m_viewer->showNextImage( item->url().path() );
-    m_slideTimer->start( kdata->slideDelay );
+    m_viewer->showNextImage( item->url() );
+    if(kdata->slideDelay)
+        m_slideTimer->start( kdata->slideDelay );
 }
 
 
@@ -604,7 +778,7 @@ void KuickShow::slotPrint()
     ImageWindow *iw = new ImageWindow( 0, id, this, "printing image" );
     KFileItem *item;
     while ( (item = it.current()) ) {
-        if (FileWidget::isImage( item ) && iw->loadImage( item->url().path()))
+        if (FileWidget::isImage( item ) && iw->loadImage( item->url() ))
             iw->printImage();
         ++it;
     }
@@ -622,6 +796,11 @@ void KuickShow::slotShowInSameWindow()
     showImage( fileWidget->getCurrentItem( false ), false );
 }
 
+void KuickShow::slotShowFullscreen()
+{
+    showImage( fileWidget->getCurrentItem( false ), false, true );
+}
+
 void KuickShow::slotDropped( const KFileItem *, QDropEvent *, const KURL::List &urls)
 {
     KURL::List::ConstIterator it = urls.begin();
@@ -647,32 +826,15 @@ void KuickShow::slotAdvanceImage( ImageWindow *view, int steps )
     KFileItem *item      = 0L; // to be shown
     KFileItem *item_next = 0L; // to be cached
 
+    if ( steps == 0 )
+        return;
+
     // the viewer might not be available yet. Factor this out somewhen.
     if ( !fileWidget ) {
         if ( m_delayedRepeatItem )
             return;
 
-        m_delayedRepeatItem = new DelayedRepeatEvent( view, steps );
-
-        KURL start;
-        QFileInfo fi( view->filename() );
-        start.setPath( fi.dirPath( true ) );
-        initGUI( start );
-
-        // see eventFilter() for explanation and similar code
-        if ( fileWidget->dirLister()->isFinished() &&
-             fileWidget->dirLister()->rootItem() )
-        {
-            fileWidget->setCurrentItem( fi.fileName() );
-            QTimer::singleShot( 0, this, SLOT( slotReplayAdvance()));
-        }
-        else
-        {
-            fileWidget->setInitialItem( fi.fileName() );
-            connect( fileWidget, SIGNAL( finished() ),
-                     SLOT( slotReplayAdvance() ));
-        }
-
+        delayAction(new DelayedRepeatEvent( view, DelayedRepeatEvent::AdvanceViewer, new int(steps) ));
         return;
     }
 
@@ -689,22 +851,26 @@ void KuickShow::slotAdvanceImage( ImageWindow *view, int steps )
     }
 
     if ( FileWidget::isImage( item ) ) {
-        QString filename;
-        KIO::NetAccess::download(item->url(), filename, this);
-        view->showNextImage( filename );
-        if (m_slideTimer->isActive())
+//        QString filename;
+//        KIO::NetAccess::download(item->url(), filename, this);
+        view->showNextImage( item->url() );
+        if (m_slideTimer->isActive() && kdata->slideDelay)
             m_slideTimer->start( kdata->slideDelay );
 
-        if ( kdata->preloadImage && item_next && item_next->url().isLocalFile() ) // preload next image
+        if ( kdata->preloadImage && item_next ) { // preload next image
             if ( FileWidget::isImage( item_next ) )
-                view->cacheImage( item_next->url().path() ); // ###
+                view->cacheImage( item_next->url() );
+        }
+        
     }
 }
 
 bool KuickShow::eventFilter( QObject *o, QEvent *e )
 {
     if ( m_delayedRepeatItem ) // we probably need to install an eventFilter over
+	{
         return true;    // kapp, to make it really safe
+	}
 
     bool ret = false;
     int eventType = e->type();
@@ -716,6 +882,7 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
         if ( KStdAccel::quit().contains( KKey( k ) ) ) {
             saveSettings();
             deleteAllViewers();
+            FileCache::shutdown();
             ::exit(0);
         }
         else if ( KStdAccel::help().contains( KKey( k ) ) ) {
@@ -726,6 +893,7 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
 
 
     ImageWindow *window = dynamic_cast<ImageWindow*>( o );
+
     if ( window ) {
         // The XWindow used to display Imlib's image is being resized when
         // switching images, causing enter- and leaveevents for this
@@ -747,14 +915,15 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
             // is used for zooming in the imagewindow
             // Key_Alt shouldn't either - otherwise Alt+F4 doesn't work, the
             // F4 gets eaten (by NetAccess' modal dialog maybe?)
+
             if ( !fileWidget )
             {
                 if ( key != Key_Escape && key != Key_Shift && key != Key_Alt )
                 {
-                    KURL start;
-                    QFileInfo fi( m_viewer->filename() );
-                    start.setPath( fi.dirPath( true ) );
-                    initGUI( start );
+                    KuickFile *file = m_viewer->currentFile();
+//                    QFileInfo fi( m_viewer->filename() );
+//                    start.setPath( fi.dirPath( true ) );
+                    initGUI( file->url().upURL() );
 
                     // the fileBrowser will list the start-directory
                     // asynchronously so we can't immediately continue. There
@@ -774,15 +943,21 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
                     // signal.
 
                     // see slotAdvanceImage() for similar code
-                    if ( fileWidget->dirLister()->isFinished() &&
-                         fileWidget->dirLister()->rootItem() )
+                    if ( fileWidget->dirLister()->isFinished() )
                     {
-                        fileWidget->setCurrentItem( fi.fileName() );
-                        QTimer::singleShot( 0, this, SLOT( slotReplayEvent()));
+                        if ( fileWidget->dirLister()->rootItem() )
+                        {
+                            fileWidget->setCurrentItem( file->url().fileName() );
+                            QTimer::singleShot( 0, this, SLOT( slotReplayEvent()));
+                        }
+                        else // finished, but no root-item -- probably an error, kill repeat-item!
+                        {
+                            abortDelayedEvent();
+                        }
                     }
-                    else
+                    else // not finished yet
                     {
-                        fileWidget->setInitialItem( fi.fileName() );
+                        fileWidget->setInitialItem( file->url().fileName() );
                         connect( fileWidget, SIGNAL( finished() ),
                                  SLOT( slotReplayEvent() ));
                     }
@@ -810,6 +985,7 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
 
             else if ( fileWidget->actionCollection()->action("delete")->shortcut().contains( key ))
             {
+                kdDebug() << "WOW, deletion happens here!" << endl;
 //      KFileItem *cur = fileWidget->getCurrentItem( false );
                 (void) fileWidget->getCurrentItem( false );
                 item = fileWidget->getNext( false ); // don't move
@@ -838,13 +1014,14 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
 
 
             if ( FileWidget::isImage( item ) ) {
-                QString filename;
-                KIO::NetAccess::download(item->url(), filename, this);
-                m_viewer->showNextImage( filename );
+//                QString filename;
+//                KIO::NetAccess::download(item->url(), filename, this);
+                m_viewer->showNextImage( item->url() );
 
-                if ( kdata->preloadImage && item_next && item_next->url().isLocalFile() ) // preload next image
+                if ( kdata->preloadImage && item_next ) { // preload next image
                     if ( FileWidget::isImage( item_next ) )
-                        m_viewer->cacheImage( item_next->url().path() ); // ###
+                        m_viewer->cacheImage( item_next->url() );
+                }
 
                 ret = true; // don't pass keyEvent
             }
@@ -862,10 +1039,10 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
                 {
                     if ( !fileWidget )
                     {
-                        KURL start;
-                        QFileInfo fi( m_viewer->filename() );
-                        start.setPath( fi.dirPath( true ) );
-                        initGUI( start );
+//                        KURL start;
+//                        QFileInfo fi( window->filename() );
+//                        start.setPath( fi.dirPath( true ) );
+                        initGUI( window->currentFile()->url().fileName() );
                     }
                     show();
                     raise();
@@ -887,7 +1064,6 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
     return KMainWindow::eventFilter( o, e );
 }
 
-
 void KuickShow::configuration()
 {
     if ( !m_accel ) {
@@ -972,30 +1148,43 @@ void KuickShow::readProperties( KConfig *kc )
         fileWidget->clearHistory();
     }
 
+    const KURL& listedURL = fileWidget->url();
     QStringList images = kc->readPathListEntry( "Images shown" );
     QStringList::Iterator it;
+    bool hasCurrentURL = false;
+
     for ( it = images.begin(); it != images.end(); ++it ) {
         KFileItem item( KFileItem::Unknown, KFileItem::Unknown, KURL::fromPathOrURL( *it ), false );
         if ( item.isReadable() )
-            showImage( &item, true );
+            if ( showImage( &item, true ) ) {
+	        // Set the current URL in the file widget, if possible
+                if ( !hasCurrentURL && listedURL.isParentOf( item.url() ))
+                    fileWidget->setInitialItem( item.url().fileName() );
+		    hasCurrentURL = true;
+	    }
     }
 
-    if ( !s_viewers.isEmpty() ) {
-        bool visible = kc->readBoolEntry( "Browser visible", true );
-        if ( !visible )
-            hide();
-    }
+    bool visible = kc->readBoolEntry( "Browser visible", false );
+    if ( visible || s_viewers.isEmpty() )
+        show();
 }
 
 void KuickShow::saveProperties( KConfig *kc )
 {
-    kc->writePathEntry( "CurrentDirectory", fileWidget->url().url() );
-    kc->writeEntry( "Browser visible", fileWidget->isVisible() );
+    kc->writeEntry( "Browser visible", fileWidget && fileWidget->isVisible() );
+    if (fileWidget)
+        kc->writePathEntry( "CurrentDirectory", fileWidget->url().url() );
 
     QStringList urls;
     QValueListIterator<ImageWindow*> it;
     for ( it = s_viewers.begin(); it != s_viewers.end(); ++it )
-        urls.append( (*it)->filename() );
+    {
+        const KURL& url = (*it)->currentFile()->url();
+        if ( url.isLocalFile() )
+            urls.append( url.path() );
+        else
+            urls.append( url.prettyURL() ); // ### check if writePathEntry( prettyURL ) works!
+    }
 
     kc->writePathEntry( "Images shown", urls );
 }
@@ -1011,7 +1200,7 @@ void KuickShow::saveSettings()
         kc->writeEntry( "OpenImagesInActiveWindow", oneWindowAction->isChecked() );
 
     if ( fileWidget ) {
-        kc->writePathEntry( "CurrentDirectory", fileWidget->url().url() );
+        kc->writePathEntry( "CurrentDirectory", fileWidget->url().prettyURL() ); // ### was url().url()
         fileWidget->writeConfig( kc, "Filebrowser" );
     }
 
@@ -1019,13 +1208,10 @@ void KuickShow::saveSettings()
 }
 
 
-void KuickShow::messageCantLoadImage( const QString& filename )
+void KuickShow::messageCantLoadImage( const KuickFile *, const QString& message )
 {
     m_viewer->clearFocus();
-    QString tmp = i18n("Unable to load the image %1.\n"
-                       "Perhaps the file format is unsupported or "
-                       "your Imlib is not installed properly.").arg(filename);
-    KMessageBox::sorry( m_viewer, tmp, i18n("Image Error") );
+    KMessageBox::information( m_viewer, message, i18n("Error"), "kuick_cant_load_image" );
 }
 
 void KuickShow::initImlib()
@@ -1040,7 +1226,7 @@ void KuickShow::initImlib()
 
         qWarning("*** KuickShow: Whoops, can't initialize imlib, trying my own palettefile now.");
         QString paletteFile = locate( "data", "kuickshow/im_palette.pal" );
-        // FIXME - does the qstrdup() cure the segfault in imlib eventually?
+        // ### - does the qstrdup() cure the segfault in imlib eventually?
         char *file = qstrdup( paletteFile.local8Bit() );
         par.palettefile = file;
         par.flags |= PARAMS_PALETTEFILE;
@@ -1056,6 +1242,7 @@ void KuickShow::initImlib()
                                "The program will now quit.");
             KMessageBox::error( this, tmp, i18n("Fatal Imlib Error") );
 
+            FileCache::shutdown();
             ::exit(1);
         }
     }
@@ -1064,15 +1251,20 @@ void KuickShow::initImlib()
 
 void KuickShow::initImlibParams( ImData *idata, ImlibInitParams *par )
 {
-    par->flags = ( PARAMS_REMAP |
+    par->flags = ( PARAMS_REMAP | PARAMS_VISUALID | PARAMS_SHAREDMEM | PARAMS_SHAREDPIXMAPS |
                    PARAMS_FASTRENDER | PARAMS_HIQUALITY | PARAMS_DITHER |
                    PARAMS_IMAGECACHESIZE | PARAMS_PIXMAPCACHESIZE );
 
+    Visual* defaultvis = DefaultVisual(x11Display(), x11Screen());
+
     par->paletteoverride = idata->ownPalette  ? 1 : 0;
     par->remap           = idata->fastRemap   ? 1 : 0;
     par->fastrender      = idata->fastRender  ? 1 : 0;
     par->hiquality       = idata->dither16bit ? 1 : 0;
     par->dither          = idata->dither8bit  ? 1 : 0;
+    par->sharedmem       = 1;
+    par->sharedpixmaps   = 1;
+    par->visualid	 = defaultvis->visualid;
     uint maxcache        = idata->maxCache;
 
     // 0 == no cache
@@ -1090,6 +1282,12 @@ void KuickShow::delayedRepeatEvent( ImageWindow *w, QKeyEvent *e )
     m_delayedRepeatItem = new DelayedRepeatEvent( w, new QKeyEvent( *e ) );
 }
 
+void KuickShow::abortDelayedEvent()
+{
+    delete m_delayedRepeatItem;
+    m_delayedRepeatItem = 0L;
+}
+
 void KuickShow::slotReplayEvent()
 {
     disconnect( fileWidget, SIGNAL( finished() ),
@@ -1111,17 +1309,8 @@ void KuickShow::slotReplayEvent()
     // --------------------------------------------------------------
 }
 
-void KuickShow::slotReplayAdvance()
+void KuickShow::replayAdvance(DelayedRepeatEvent *event)
 {
-    if ( !m_delayedRepeatItem )
-        return;
-
-    disconnect( fileWidget, SIGNAL( finished() ),
-                this, SLOT( slotReplayAdvance() ));
-
-    DelayedRepeatEvent *e = m_delayedRepeatItem;
-    m_delayedRepeatItem = 0L; // otherwise, eventFilter aborts
-
     // ### WORKAROUND for QIconView bug in Qt <= 3.0.3 at least
     // Sigh. According to qt-bugs, they won't fix this bug ever. So you can't
     // rely on sorting to be correct before the QIconView has been show()n.
@@ -1133,8 +1322,62 @@ void KuickShow::slotReplayAdvance()
     }
     // --------------------------------------------------------------
 
-    slotAdvanceImage( e->viewer, e->steps );
-    delete e;
+    slotAdvanceImage( event->viewer, *(int *) (event->data) );
+}
+
+void KuickShow::delayAction(DelayedRepeatEvent *event)
+{
+    if (m_delayedRepeatItem)
+        return;
+
+    m_delayedRepeatItem = event;
+
+    KURL url = event->viewer->currentFile()->url();
+//    QFileInfo fi( event->viewer->filename() );
+//    start.setPath( fi.dirPath( true ) );
+    initGUI( url.upURL() );
+
+    // see eventFilter() for explanation and similar code
+    if ( fileWidget->dirLister()->isFinished() &&
+         fileWidget->dirLister()->rootItem() )
+    {
+        fileWidget->setCurrentItem( url.fileName() );
+        QTimer::singleShot( 0, this, SLOT( doReplay()));
+    }
+    else
+    {
+        fileWidget->setInitialItem( url.fileName() );
+        connect( fileWidget, SIGNAL( finished() ),
+                 SLOT( doReplay() ));
+    }
+}
+
+void KuickShow::doReplay()
+{
+    if (!m_delayedRepeatItem)
+        return;
+
+    disconnect( fileWidget, SIGNAL( finished() ),
+                this, SLOT( doReplay() ));
+
+    switch (m_delayedRepeatItem->action)
+    {
+        case DelayedRepeatEvent::DeleteCurrentFile:
+            performDeleteCurrentImage((QWidget *) m_delayedRepeatItem->data);
+            break;
+        case DelayedRepeatEvent::TrashCurrentFile:
+            performTrashCurrentImage((QWidget *) m_delayedRepeatItem->data);
+            break;
+        case DelayedRepeatEvent::AdvanceViewer:
+            replayAdvance(m_delayedRepeatItem);
+            break;
+        default:
+            kdWarning() << "doReplay: unknown action -- ignoring: " << m_delayedRepeatItem->action << endl;
+            break;
+    }
+
+    delete m_delayedRepeatItem;
+    m_delayedRepeatItem = 0L;
 }
 
 void KuickShow::toggleBrowser()
